# Archived

During a recent refactoring, a lot of code that obsoleted. I am temporarily
archiving them just in case I want to reference them later. Most likely, they
will be removed at some point in the future.

This also contains some obsoleted sections from the README.md.

### Code Generation to Use DigitalWriteFast

Looking at the CPU cycles in the __Resource Consumption__ section below, the
time taken for a single `Renderer::renderField()` can range from 80 to 204
microseconds. A significant contribution to the CPU cycles is the slow
implementation of
[digitalWrite() in Arduino](https://forum.arduino.cc/index.php?topic=46896.0).
A faster version called
[digitalWriteFast](https://github.com/NicksonYap/digitalWriteFast)
is available on GitHub based on the Forum discussions.

I wrote a Python script called `./tools/fast_driver.py` which generates C++ code
for a subclass of `Driver` that uses the `digitalWriteFast()`. The
script is called like this:
```
$ ./tools/fast_driver.py --digit_pins 12 14 15 16 \
        --segment_direct_pins 4 5 6 7 8 9 10 11 \
        --class_name FastDirectDriver --output_files
```
which generates two files in the current directory:
```
FastDirectDriver.h
FastDirectDriver.cpp
```

These generated classes will replace the class generated by `DriverBuilder`:
```
Driver* driver = DriverBuilder(hardware)
    .setDimmablePatterns(dimmablePatterns)
    .setNumDigits(NUM_DIGITS)
    .useModulation(NUM_SUBFIELDS)
    ...
    .build();
```
with just
```
Driver* driver = new FastDirectDriver(
    dimmablePatterns, NUM_DIGITS, NUM_SUBFIELDS);
```

The generated code has no dependency to `Hardware`, it writes directly to the
`digitalWriteFast()` method (which is actually implemented as macros). The
resulting generated code in all configurations runs `renderField()` between
76-84 microseconds, which is 1.3 to 2.7 times faster than the `Driver` versions
created by `DriverBuilder`.

For "production" code that uses the AceSegment library, the code generation is
the recommended procedure. See the example code in `examples/AceSegmentDemo/`
for more details.

## CPU Time

For the fastest rendering time, a Python script in the `./tools` directory
uses code generation to create subclasses of `Renderer` that uses
[digitalWriteFast()](https://github.com/NicksonYap/digitalWriteFast) routines
which are 10-20X faster than the default `digitalWrite()` methods in Arduino.
The generated code makes `Driver::renderField()` consume 25% to 65% fewer CPU
cycles (i.e. 1.3 to 2.7 times faster).

